@@ -43,25 +43,17 @@ ply_to_numpy_dtype = {
 numpy_to_ply_dtype = {np.dtype(v): k for k, v in ply_to_numpy_dtype.items()}
 
 
-_cell_type_to_count = {"vertex": 1, "line": 2, "triangle": 3, "quad": 4}
-
-
-def cell_type_to_count(cell_type):
-    if cell_type in _cell_type_to_count:
-        return _cell_type_to_count[cell_type]
-
-    match = re.fullmatch(r"polygon(\d+)", cell_type)
-    if match:
-        return int(match.group(1))
-
-
-_cell_type_from_count = {val: key for (key, val) in _cell_type_to_count.items()}
-
-
 def cell_type_from_count(count):
-    """Reverse of ``cell_type_to_count``, defaults to ``"polygon" + str(counr)``
-    if unknown."""
-    return _cell_type_from_count.get(count) or "polygon" + str(count)
+    if count == 1:
+        return "vertex"
+    elif count == 2:
+        return "line"
+    elif count == 3:
+        return "triangle"
+    elif count == 4:
+        return "quad"
+
+    return "polygon"
 
 
 def read(filename):
@@ -73,7 +65,7 @@ def read(filename):
 def _next_line(f):
     # fast forward to the next significant line
     while True:
-        line = f.readline().decode("utf-8").strip()
+        line = f.readline().decode().strip()
         if line and line[:7] != "comment":
             break
     return line
@@ -81,7 +73,7 @@ def _next_line(f):
 
 def read_buffer(f):
     # assert that the first line reads `ply`
-    line = f.readline().decode("utf-8").strip()
+    line = f.readline().decode().strip()
     if line != "ply":
         raise ReadError("Expected ply")
 
@@ -145,7 +137,6 @@ def read_buffer(f):
                 f"got `{line}`"
             )
 
-    # read header
     if is_binary:
         mesh = _read_binary(
             f,
@@ -180,6 +171,8 @@ def _read_ascii(
     cell_data_names,
     cell_dtypes,
 ):
+    assert len(cell_data_names) == len(cell_dtypes)
+
     # assert that all formats are the same
     # Now read the data
     dtype = np.dtype(
@@ -210,16 +203,16 @@ def _read_ascii(
     }
     cell_data = {}
 
-    # the faces must be read line-by-line
-    polygons = collections.defaultdict(list)
+    cell_blocks = []
+
     for k in range(num_cells):
-        line = f.readline().decode("utf-8").strip()
+        line = f.readline().decode().strip()
         data = line.split()
+
         if k == 0:
             # initialize the cell data arrays
             i = 0
             cell_data = {}
-            assert len(cell_data_names) == len(cell_dtypes)
             for name, dtype in zip(cell_data_names, cell_dtypes):
                 if name == "vertex_indices":
                     n = int(data[i])
@@ -229,12 +222,18 @@ def _read_ascii(
                     cell_data[name] = collections.defaultdict(list)
                     i += 1
 
+        # go over the line
         i = 0
         for name, dtype in zip(cell_data_names, cell_dtypes):
             if name == "vertex_indices":
-                n = int(data[i])
-                dtype = ply_to_numpy_dtype[dtype[1]]
-                polygons[n].append([dtype(data[j]) for j in range(i + 1, i + n + 1)])
+                idx_dtype, value_dtype = dtype
+                n = ply_to_numpy_dtype[idx_dtype](data[i])
+                dtype = ply_to_numpy_dtype[value_dtype]
+                idx = dtype(data[i + 1 : i + n + 1])
+                if len(cell_blocks) == 0 or len(cell_blocks[-1].data[-1]) != n:
+                    cell_blocks.append(CellBlock(cell_type_from_count(n), [idx]))
+                else:
+                    cell_blocks[-1].data.append(idx)
                 i += n + 1
             else:
                 dtype = ply_to_numpy_dtype[dtype]
@@ -242,15 +241,11 @@ def _read_ascii(
                 cell_data[name][n] += [dtype(data[j]) for j in range(i, i + 1)]
                 i += 1
 
-    cells = [
-        CellBlock(cell_type_from_count(n), np.array(data))
-        for (n, data) in polygons.items()
-    ]
     cell_data = {
         key: [np.array(v) for v in value.values()] for key, value in cell_data.items()
     }
 
-    return Mesh(verts, cells, point_data=point_data, cell_data=cell_data)
+    return Mesh(verts, cell_blocks, point_data=point_data, cell_data=cell_data)
 
 
 def _read_binary(
@@ -395,18 +390,18 @@ def write(filename, mesh, binary=True):  # noqa: C901
         fh.write(b"ply\n")
 
         if binary:
-            fh.write(f"format binary_{sys.byteorder}_endian 1.0\n".encode("utf-8"))
+            fh.write(f"format binary_{sys.byteorder}_endian 1.0\n".encode())
         else:
             fh.write(b"format ascii 1.0\n")
 
         fh.write(
             "comment Created by meshio v{}, {}\n".format(
                 __version__, datetime.datetime.now().isoformat()
-            ).encode("utf-8")
+            ).encode()
         )
 
         # counts
-        fh.write(f"element vertex {mesh.points.shape[0]:d}\n".encode("utf-8"))
+        fh.write(f"element vertex {mesh.points.shape[0]:d}\n".encode())
         #
         dim_names = ["x", "y", "z"]
         # From <https://en.wikipedia.org/wiki/PLY_(file_format)>:
@@ -429,7 +424,7 @@ def write(filename, mesh, binary=True):  # noqa: C901
         }
         for k in range(mesh.points.shape[1]):
             type_name = type_name_table[mesh.points.dtype]
-            fh.write(f"property {type_name} {dim_names[k]}\n".encode("utf-8"))
+            fh.write(f"property {type_name} {dim_names[k]}\n".encode())
 
         pd = []
         for key, value in mesh.point_data.items():
@@ -441,17 +436,20 @@ def write(filename, mesh, binary=True):  # noqa: C901
                 )
                 continue
             type_name = type_name_table[value.dtype]
-            fh.write(f"property {type_name} {key}\n".encode("utf-8"))
+            fh.write(f"property {type_name} {key}\n".encode())
             pd.append(value)
 
         num_cells = 0
+        legal_cell_types = ["vertex", "line", "triangle", "quad", "polygon"]
         for cell_type, c in mesh.cells:
-            if cell_type_to_count(cell_type):
+            if cell_type in legal_cell_types:
                 num_cells += c.data.shape[0]
+
         if num_cells > 0:
-            fh.write(f"element face {num_cells:d}\n".encode("utf-8"))
+            fh.write(f"element face {num_cells:d}\n".encode())
 
             # possibly cast down to int32
+            # TODO don't alter the mesh data
             has_cast = False
             for k, (cell_type, data) in enumerate(mesh.cells):
                 if data.dtype == np.int64:
@@ -473,11 +471,7 @@ def write(filename, mesh, binary=True):  # noqa: C901
 
             if cell_dtype is not None:
                 ply_type = numpy_to_ply_dtype[cell_dtype]
-                fh.write(
-                    "property list {} {} vertex_indices\n".format(
-                        "uint8", ply_type
-                    ).encode("utf-8")
-                )
+                fh.write(f"property list uint8 {ply_type} vertex_indices\n".encode())
 
         # TODO other cell data
         fh.write(b"end_header\n")
@@ -489,7 +483,7 @@ def write(filename, mesh, binary=True):  # noqa: C901
 
             # cells
             for cell_type, data in mesh.cells:
-                if cell_type_to_count(cell_type) is None:
+                if cell_type not in legal_cell_types:
                     warnings.warn(
                         'cell_type "{}" is not supported by ply format - skipping'
                     )
@@ -509,7 +503,7 @@ def write(filename, mesh, binary=True):  # noqa: C901
             out = np.rec.fromarrays([coord for coord in mesh.points.T] + pd)
             fmt = " ".join(["{}"] * len(out[0]))
             out = "\n".join([fmt.format(*row) for row in out]) + "\n"
-            fh.write(out.encode("utf-8"))
+            fh.write(out.encode())
 
             # cells
             for cell_type, data in mesh.cells:
@@ -522,7 +516,7 @@ def write(filename, mesh, binary=True):  # noqa: C901
                 # np.savetxt(fh, out, "%d  %d %d %d")
                 fmt = " ".join(["{}"] * out.shape[1])
                 out = "\n".join([fmt.format(*row) for row in out]) + "\n"
-                fh.write(out.encode("utf-8"))
+                fh.write(out.encode())
 
 
 register("ply", [".ply"], read, {"ply": write})
